ENCODING = 'utf-8'
PRESENCE = 'presence'
EXIT = 'exit'
TIME = 'time'
ACTION = 'action'
ANSWER = 'answer'
TEXT = 'text'
RESPONSE = 'response'
MESSAGE = 'message'
GET_CONTACT = 'get_contacts'
CONTACTS = 'alert'
ADD_CONTACT = 'add_contacts'
DEL_CONTACT = 'del_contacts'
CONTACT_NAME = 'contact_name'
FROM = 'from'
TO = 'to'
NICKNAME = 'nickname'
ERROR = 'error'
DEFAULT_PORT = 7777
DEFAULT_IP = '127.0.0.1'
MAX_DATA_LENGTH = 1024
DATABASE_SERVER = 'sqlite:///database_server.db3'
DATABASE_CLIENT = 'sqlite:///database_client.db3'
PUBLIC_KEY = 'public_key'
DATA = 'bin'
ACCESS = 'access'
